package com.ekexiu.project.platform.views.dao;

import com.ekexiu.project.platform.views.pojo.ArticleSum;
import com.ekexiu.project.platform.views.pojo.DayData;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.orm.annotation.dao.DAO;
import org.jfw.apt.orm.annotation.dao.method.OrderBy;
import org.jfw.apt.orm.annotation.dao.method.operator.PageQuery;
import org.jfw.apt.orm.annotation.dao.param.Alias;
import org.jfw.apt.orm.annotation.dao.param.GtEq;
import org.jfw.apt.orm.annotation.dao.param.Like;
import org.jfw.apt.orm.annotation.dao.param.LtEq;
import org.jfw.util.PageQueryResult;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.LinkedList;
import java.util.List;

@DAO
public abstract class DayDao {
	public List<DayData> query(Connection con, String tablePrefix, String[] srcs, String[] ids, String bt, String et, String sortStr) throws SQLException {
		StringBuilder sb = new StringBuilder();
		boolean hasSource = srcs != null && srcs.length > 0;
		boolean hasid = ids != null && ids.length > 0;
		sb.append("SELECT ID,DAY,SOURCE,PV FROM ").append(tablePrefix).append("_DAY WHERE DAY>=? AND DAY<=?");
		if (hasSource) {
			sb.append(" AND SOURCE IN(");
			for (int i = 0; i < srcs.length; ++i) {
				sb.append("?,");
			}
			sb.setCharAt(sb.length() - 1, ')');
		}
		if (hasid) {
			sb.append(" AND ID IN(");
			for (int i = 0; i < ids.length; ++i) {
				sb.append("?,");
			}
			sb.setCharAt(sb.length() - 1, ')');
		}
		if (sortStr != null && sortStr.trim().length() > 0) {
			sb.append(" ORDER BY ").append(sortStr);
		}
		int idx = 1;
		java.sql.PreparedStatement ps = con.prepareStatement(sb.toString());
		try {
			ps.setString(idx++, bt);
			ps.setString(idx++, et);
			if (hasSource) {
				for (int i = 0; i < srcs.length; ++i) {
					ps.setString(idx++, srcs[i]);
				}
			}
			if (hasid) {
				for (int i = 0; i < ids.length; ++i) {
					ps.setString(idx++, ids[i]);
				}
			}
			List<DayData> ret = new LinkedList<DayData>();
			java.sql.ResultSet rs = ps.executeQuery();
			try {
				while (rs.next()) {
					DayData dd = new DayData();
					dd.setId(rs.getString(1));
					dd.setDay(rs.getString(2));
					dd.setSource(rs.getString(3));
					dd.setPv(rs.getInt(4));
					ret.add(dd);
				}
				return ret;
			} finally {
				try {
					rs.close();
				} catch (Exception _m_2) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception _m_3) {
			}
		}
	}

	public List<DayData> query(Connection con, String tablePrefix, String id, String bt, String et) throws SQLException {
		StringBuilder sb = new StringBuilder();
		sb.append("SELECT ID,DAY,SOURCE,PV FROM ").append(tablePrefix).append("_DAY WHERE DAY>=? AND DAY<=? AND ID=? ORDER BY DAY");
		java.sql.PreparedStatement ps = con.prepareStatement(sb.toString());
		try {
			ps.setString(1, bt);
			ps.setString(2, et);
			ps.setString(3, id);
			List<DayData> ret = new LinkedList<DayData>();
			java.sql.ResultSet rs = ps.executeQuery();
			try {
				while (rs.next()) {
					DayData dd = new DayData();
					dd.setId(rs.getString(1));
					dd.setDay(rs.getString(2));
					dd.setSource(rs.getString(3));
					dd.setPv(rs.getInt(4));
					ret.add(dd);
				}
				return ret;
			} finally {
				try {
					rs.close();
				} catch (Exception _m_2) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception _m_3) {
			}
		}
	}

	public List<DayData> query(Connection con,String tablePrefix,String bt,String et,@Nullable String[] srcs,int rows) throws SQLException{		
		boolean hasSource = srcs!=null && srcs.length>0;
		StringBuilder sb = new StringBuilder();
		sb.append("SELECT ID,SUM(PV) PV FROM ").append(tablePrefix).append("_DAY WHERE DAY>=? AND DAY<=?");
		if(hasSource){
			sb.append(" AND SOURCE IN (");
			for(int i = 0; i < srcs.length; ++i){
				sb.append("?,");
			}
			sb.setCharAt(sb.length()-1, ')');
		}
		sb.append(" GROUP BY ID ORDER BY PV DESC LIMIT ?");
	    java.sql.PreparedStatement ps = con.prepareStatement(sb.toString());
        try{
        	int idx=1;
            ps.setString(idx++,bt);
            ps.setString(idx++,et);
            if(hasSource){
            	for(String src:srcs)
            	ps.setString(idx++, src);
            }
            ps.setInt(idx++,rows);
            List<DayData> ret = new LinkedList<DayData>();
            java.sql.ResultSet rs = ps.executeQuery();
            try{
            	while(rs.next()){
            		DayData dd = new DayData();
            		dd.setId(rs.getString(1));
            		dd.setPv(rs.getInt(2));
            		ret.add(dd);
            	}
            	return ret;
            }finally{
                try{rs.close();}catch(Exception _m_2){}
            }
        }finally{
            try{ps.close();}catch(Exception _m_3){}
        }
	}

	@PageQuery
	@OrderBy("ORDER BY a.CREATE_TIME DESC")
	public abstract PageQueryResult<ArticleSum> queryByTime(Connection con, @Nullable String catalog, @Nullable @Like String title, @Nullable @Like String source, @Nullable String creator, @Nullable Boolean published, @Nullable @GtEq @Alias("modifyTime") String bt, @Nullable @Alias("modifyTime") @LtEq String et, int pageSize, int pageNo) throws SQLException;

	@PageQuery
	@OrderBy("ORDER BY ads.sum DESC")
	public abstract PageQueryResult<ArticleSum> queryByViews(Connection con, @Nullable String catalog, @Nullable @Like String title, @Nullable @Like String source, @Nullable String creator, @Nullable Boolean published, @Nullable @GtEq @Alias("modifyTime") String bt, @Nullable @Alias("modifyTime") @LtEq String et, int pageSize, int pageNo)throws SQLException;


}
